﻿using HeuristicLab.Common;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Algorithms.DataAnalysis.FastFunctionExtraction {
    internal static class Utils {
        // return the nth row of a matrix
        public static T[] GetRow<T>(this T[,] matrix, int n) {
            var columns = matrix.GetLength(1);
            var array = new T[columns];
            for (int i = 0; i < columns; ++i)
                array[i] = matrix[n, i];
            return array;
        }

        public static IEnumerable<int> FindAllIndices<T>(this IEnumerable<T> values, Predicate<T> match) {
            return values.Select((b, i) => match(b) ? i : -1).Where(i => i != -1).ToArray();
        }

        public static int[] Argsort<T>(T[] vals) where T : IComparable {
            var indices = Enumerable.Range(0, vals.Length).ToArray();
            Array.Sort(indices, (idx1, idx2) => vals[idx1].CompareTo(vals[idx2]));
            return indices;
        }

        // find the array containing n (linearly) evenly spaced double values ranging from start to end (including both)
        public static IEnumerable<double> Linspace(double start, double end, int n) {
            if (end < start) throw new ArgumentException("end must be higher that start");
            if (n <= 0) return Enumerable.Empty<double>();
            if (n == 1) return new double[1] { start };
            var range = end - start;
            return Enumerable.Range(0, n).Select(idx => start + range * idx / (n - 1));
        }

        public static double[] Normalize(double[] vals)
            => Bias(vals, 0, 1);

        public static double[] Bias(double[] vals, double targetAvg, double targetStd) {
            var result = new double[vals.Length];

            // bias standard deviation
            var stdRatio = vals.StandardDeviationPop() / targetStd;
            for (int i = 0; i < vals.Length; i++) {
                result[i] = vals[i] / stdRatio;
            }

            // bias average
            var avgDiff = result.Average() - targetAvg;
            for (int i = 0; i < vals.Length; i++) {
                result[i] = result[i] - avgDiff;
            }

            return result;
        }

        public static bool[] ToBoolArray(this int val, int min_length) {
            var temp = Convert.ToString(val, 2).Select(s => s.Equals('1')).ToArray();
            if (temp.Length >= min_length) return temp;

            var ans = new bool[min_length];
            for(int i = 0; i < min_length - temp.Length; i++) {
                ans[i] = false;
            }
            for(int i = 0; i < temp.Length; i++) {
                ans[i + min_length - temp.Length] = temp[i];
            }
            return ans;
        }
    }
}
